@@ -35,11 +35,12 @@
 #include "battle_interface.h"
 #include "battle_arena.h"
 #include "battle_army.h"
+#include "world.h"
 #include "battle.h"
 
 namespace Battle
 {
-    void GetSummaryParams(u8 res1, u8 res2, const std::string &, u32 exp, ICN::icn_t &, std::string &);
+    void GetSummaryParams(u8 res1, u8 res2, const HeroBase &, u32 exp, ICN::icn_t &, std::string &);
     void SpeedRedraw(const Point &);
 }
 
@@ -157,7 +158,7 @@ void Battle::DialogBattleSettings(void)
     display.Flip();
 }
 
-void Battle::GetSummaryParams(u8 res1, u8 res2, const std::string & name, u32 exp, ICN::icn_t & icn_anim, std::string & msg)
+void Battle::GetSummaryParams(u8 res1, u8 res2, const HeroBase & hero, u32 exp, ICN::icn_t & icn_anim, std::string & msg)
 {
     if(res1 & RESULT_WINS)
     {
@@ -169,30 +170,34 @@ void Battle::GetSummaryParams(u8 res1, u8 res2, const std::string & name, u32 ex
     	    msg.append(_("The enemy has fled!"));
 	else
 	    msg.append(_("A glorious victory!"));
-    	msg.append("\n");
-    	msg.append(_("For valor in combat, %{name} receives %{exp} experience"));
-    	String::Replace(msg, "%{name}", name);
-    	String::Replace(msg, "%{exp}", exp);
+
+	if(Skill::Primary::HEROES == hero.GetType())
+	{
+    	    msg.append("\n");
+    	    msg.append(_("For valor in combat, %{name} receives %{exp} experience"));
+    	    String::Replace(msg, "%{name}", hero.GetName());
+    	    String::Replace(msg, "%{exp}", exp);
+    	}
     }
     else
     if(res1 & RESULT_RETREAT)
     {
 	icn_anim = ICN::CMBTFLE3;
 	msg.append(_("The cowardly %{name} flees from battle."));
-    	String::Replace(msg, "%{name}", name);
+    	String::Replace(msg, "%{name}", hero.GetName());
     }
     else
     if(res1 & RESULT_SURRENDER)
     {
 	icn_anim = ICN::CMBTSURR;
 	msg.append(_("%{name} surrenders to the enemy, and departs in shame."));
-    	String::Replace(msg, "%{name}", name);
+    	String::Replace(msg, "%{name}", hero.GetName());
     }
     else
     {
 	icn_anim = ICN::CMBTLOS3;
 	msg.append(_("Your force suffer a bitter defeat, and %{name} abandons your cause."));
-    	String::Replace(msg, "%{name}", name);
+    	String::Replace(msg, "%{name}", hero.GetName());
     }
 }
 
@@ -214,25 +219,25 @@ void Battle::Arena::DialogBattleSummary(const Result & res) const
 
     if((res.army1 & RESULT_WINS) && army1->GetCommander() && (CONTROL_HUMAN & army1->GetCommander()->GetControl()))
     {
-    	GetSummaryParams(res.army1, res.army2, army1->GetCommander()->GetName(), res.exp1, icn_anim, msg);
+    	GetSummaryParams(res.army1, res.army2, *army1->GetCommander(), res.exp1, icn_anim, msg);
 	if(conf.Music()) AGG::PlayMusic(MUS::BATTLEWIN, false);
     }
     else
     if((res.army2 & RESULT_WINS) && army2->GetCommander() && (CONTROL_HUMAN & army2->GetCommander()->GetControl()))
     {
-    	GetSummaryParams(res.army2, res.army1, army2->GetCommander()->GetName(), res.exp2, icn_anim, msg);
+    	GetSummaryParams(res.army2, res.army1, *army2->GetCommander(), res.exp2, icn_anim, msg);
 	if(conf.Music()) AGG::PlayMusic(MUS::BATTLEWIN, false);
     }
     else
     if(army1->GetCommander() && (CONTROL_HUMAN & army1->GetCommander()->GetControl()))
     {
-    	GetSummaryParams(res.army1, res.army2, army1->GetCommander()->GetName(), res.exp1, icn_anim, msg);
+    	GetSummaryParams(res.army1, res.army2, *army1->GetCommander(), res.exp1, icn_anim, msg);
 	if(conf.Music()) AGG::PlayMusic(MUS::BATTLELOSE, false);
     }
     else
     if(army2->GetCommander() && (CONTROL_HUMAN & army2->GetCommander()->GetControl()))
     {
-    	GetSummaryParams(res.army2, res.army1, army2->GetCommander()->GetName(), res.exp2, icn_anim, msg);
+    	GetSummaryParams(res.army2, res.army1, *army2->GetCommander(), res.exp2, icn_anim, msg);
 	if(conf.Music()) AGG::PlayMusic(MUS::BATTLELOSE, false);
     }
     else
@@ -357,7 +362,7 @@ void Battle::Arena::DialogBattleSummary(const Result & res) const
     display.Flip();
 }
 
-u8 Battle::Arena::DialogBattleHero(const HeroBase & hero) const
+u8 Battle::Arena::DialogBattleHero(const HeroBase & hero, bool buttons) const
 {
     Display & display = Display::Get();
     Cursor & cursor = Cursor::Get();
@@ -367,7 +372,7 @@ u8 Battle::Arena::DialogBattleHero(const HeroBase & hero) const
     cursor.Hide();
     cursor.SetThemes(Cursor::POINTER);
 
-    const bool readonly = current_color != hero.GetColor();
+    const bool readonly = current_color != hero.GetColor() || !buttons;
     const Sprite & dialog = AGG::GetICN((conf.ExtGameEvilInterface() ? ICN::VGENBKGE : ICN::VGENBKG), 0);
 
     Rect pos_rt;
@@ -469,6 +474,8 @@ u8 Battle::Arena::DialogBattleHero(const HeroBase & hero) const
 	btnSurrender.isEnable() && le.MousePressLeft(btnSurrender) ? btnSurrender.PressDraw() : btnSurrender.ReleaseDraw();
 	le.MousePressLeft(btnClose) ? btnClose.PressDraw() : btnClose.ReleaseDraw();
 
+	if(!buttons && !le.MousePressRight()) break;
+
 	if(Game::HotKeyPress(Game::EVENT_BATTLE_CASTSPELL) ||
 		(btnCast.isEnable() && le.MouseClickLeft(btnCast))) result = 1;
 
@@ -529,6 +536,28 @@ bool Battle::DialogBattleSurrender(const HeroBase & hero, u32 cost)
 
     Button btnAccept(pos_rt.x + 90, pos_rt.y + 150, icn, 0, 1);
     Button btnDecline(pos_rt.x + 295, pos_rt.y + 150, icn, 2, 3);
+    Button btnMarket(pos_rt.x + (pos_rt.w - 16) / 2, pos_rt.y + 145, (conf.ExtGameEvilInterface() ? ICN::ADVEBTNS : ICN::ADVBTNS), 4, 5);
+    const Kingdom & kingdom = world.GetKingdom(hero.GetColor());
+
+    if(! kingdom.AllowPayment(payment_t(Resource::GOLD, cost)))
+    {
+	btnAccept.Press();
+    	btnAccept.SetDisable(true);
+    }
+
+    if(kingdom.GetCountMarketplace())
+    {
+	if(kingdom.AllowPayment(payment_t(Resource::GOLD, cost)))
+    	    btnMarket.SetDisable(true);
+	else
+	{
+    	    std::string msg = _("Not enough gold (%{gold})");
+    	    String::Replace(msg, "%{gold}", cost - kingdom.GetFunds().Get(Resource::GOLD));
+	    Text text(msg, Font::SMALL);
+	    text.Blit(btnMarket.x + (btnMarket.w - text.w()) / 2, btnMarket.y - 15);
+    	    btnMarket.Draw();
+	}
+    }
 
     btnAccept.Draw();
     btnDecline.Draw();
@@ -546,18 +575,33 @@ bool Battle::DialogBattleSurrender(const HeroBase & hero, u32 cost)
     String::Replace(str, "%{price}", cost);
 
     TextBox box(str, Font::BIG, 275);
-    box.Blit(pos_rt.x + 175, pos_rt.y + 65);
-    u8 result = 0;
+    box.Blit(pos_rt.x + 175, pos_rt.y + 50);
+    bool result = false;
 
     cursor.Show();
     display.Flip();
 
     while(le.HandleEvents() && !result)
     {
+	if(btnAccept.isEnable())
 	le.MousePressLeft(btnAccept) ? btnAccept.PressDraw() : btnAccept.ReleaseDraw();
 	le.MousePressLeft(btnDecline) ? btnDecline.PressDraw() : btnDecline.ReleaseDraw();
 
-	if(le.MouseClickLeft(btnAccept)) result = 1;
+	if(btnMarket.isEnable())
+	le.MousePressLeft(btnMarket) ? btnMarket.PressDraw() : btnMarket.ReleaseDraw();
+
+	if(btnAccept.isEnable() && le.MouseClickLeft(btnAccept)) result = true;
+
+        if(btnMarket.isEnable() && le.MouseClickLeft(btnMarket))
+        {
+            Dialog::Marketplace(false);
+
+            if(kingdom.AllowPayment(payment_t(Resource::GOLD, cost)))
+	    {
+		btnAccept.Release();
+    		btnAccept.SetDisable(false);
+	    }
+        }
 
         // exit
 	if(Game::HotKeyPress(Game::EVENT_DEFAULT_EXIT) || le.MouseClickLeft(btnDecline)) break;
