@@ -20,7 +20,6 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 
-#include <bitset>
 #include "agg.h"
 #include "monster.h"
 #include "settings.h"
@@ -46,7 +45,7 @@ struct buildstats_t
 };
 
 buildstats_t _builds[] = {
-    // id                                             gold wood mercury ore sulfur crystal gems
+    // id                             gold wood mercury ore sulfur crystal gems
     { BUILD_THIEVESGUILD, Race::ALL, { 750, 5, 0, 0, 0, 0, 0 } },
     { BUILD_TAVERN,       Race::ALL, { 500, 5, 0, 0, 0, 0, 0 } },
     { BUILD_SHIPYARD,     Race::ALL, {2000,20, 0, 0, 0, 0, 0 } },
@@ -240,12 +239,21 @@ u8 GetIndexBuildingSprite(u32 build)
     return 0;
 }
 
-BuildingInfo::BuildingInfo(const Castle & c, building_t b) : castle(c), building(b), area(0, 0, 135, 57), disable(false)
+BuildingInfo::BuildingInfo(const Castle & c, building_t b) : castle(c), building(b), area(0, 0, 135, 57), bcond(ALLOW_BUILD)
 {
     if(IsDwelling()) building = castle.GetActualDwelling(b);
 
     building = castle.isBuild(b) ? castle.GetUpgradeBuilding(b) : b;
 
+    if(BUILD_TAVERN == building && Race::NECR == castle.GetRace())
+	building = Settings::Get().PriceLoyaltyVersion() ? BUILD_SHRINE : BUILD_TAVERN;
+
+    bcond = castle.CheckBuyBuilding(building);
+
+    // generate description
+    if(BUILD_DISABLE == bcond)
+	description = GetConditionDescription();
+    else
     if(IsDwelling())
     {
 	description = _("The %{building} produces %{monster}.");
@@ -253,7 +261,7 @@ BuildingInfo::BuildingInfo(const Castle & c, building_t b) : castle(c), building
         String::Replace(description, "%{monster}", String::Lower(Monster(castle.GetRace(), building).GetMultiName()));
     }
     else
-	description = Castle::GetDescriptionBuilding(building, castle.GetRace());
+    	description = Castle::GetDescriptionBuilding(building, castle.GetRace());
 
     switch(building)
     {
@@ -277,10 +285,7 @@ BuildingInfo::BuildingInfo(const Castle & c, building_t b) : castle(c), building
 	default: break;
     }
 
-    // necr and tavern check
-    if(BUILD_SHRINE == building && (Race::NECR != castle.GetRace() || !Settings::Get().PriceLoyaltyVersion()))
-	disable = true;
-
+    // fix area for capratin
     if(b == BUILD_CAPTAIN)
     {
 	const Sprite & sprite = AGG::GetICN(ICN::Get4Captain(castle.GetRace()),
@@ -306,11 +311,6 @@ const Rect & BuildingInfo::GetArea(void) const
     return area;
 }
 
-bool BuildingInfo::IsDisable(void) const
-{
-    return disable;
-}
-
 bool BuildingInfo::IsDwelling(void) const
 {
     switch(building)
@@ -342,18 +342,16 @@ void BuildingInfo::RedrawCaptain(void)
     const Sprite & sprite_deny  = AGG::GetICN(ICN::TOWNWIND, 12);
     const Sprite & sprite_money = AGG::GetICN(ICN::TOWNWIND, 13);
     Point dst_pt;
-
-    buildcond_t build_condition = castle.CheckBuyBuilding(building);
-    bool allow_buy = build_condition == ALLOW_BUILD;
+    bool allow_buy = bcond == ALLOW_BUILD;
 
     // indicator
     dst_pt.x = area.x + 65;
     dst_pt.y = area.y + 60;
-    if(castle.isBuild(building)) sprite_allow.Blit(dst_pt);
+    if(bcond == ALREADY_BUILT) sprite_allow.Blit(dst_pt);
     else
     if(! allow_buy)
     {
-	if(LACK_RESOURCES == build_condition)
+	if(LACK_RESOURCES == bcond)
 	    sprite_money.Blit(dst_pt);
 	else
 	    sprite_deny.Blit(dst_pt);
@@ -365,57 +363,68 @@ void BuildingInfo::Redraw(void)
     if(BUILD_CAPTAIN == building)
     {
 	RedrawCaptain();
-	return;
     }
-
-    u8 index = GetIndexBuildingSprite(building);
-    Display & display = Display::Get();
-
-    AGG::GetICN(ICN::BLDGXTRA, 0).Blit(area.x, area.y);
-
-    if(disable)
-    {
-	display.FillRect(0, 0, 0, Rect(area.x + 1, area.y + 1, area.w, area.h));
-	return;
-    }
-
-    AGG::GetICN(ICN::Get4Building(castle.GetRace()),
-		index).Blit(area.x + 1, area.y + 1);
-
-    const Sprite & sprite_allow = AGG::GetICN(ICN::TOWNWIND, 11);
-    const Sprite & sprite_deny  = AGG::GetICN(ICN::TOWNWIND, 12);
-    const Sprite & sprite_money = AGG::GetICN(ICN::TOWNWIND, 13);
-    Point dst_pt;
-
-    buildcond_t build_condition = castle.CheckBuyBuilding(building);
-    bool allow_buy = build_condition == ALLOW_BUILD;
-
-    // indicator
-    dst_pt.x = area.x + 115;
-    dst_pt.y = area.y + 40;
-    if(castle.isBuild(building)) sprite_allow.Blit(dst_pt);
     else
-    if(! allow_buy)
     {
-	if(LACK_RESOURCES == build_condition)
-	    sprite_money.Blit(dst_pt);
+	u8 index = GetIndexBuildingSprite(building);
+
+	if(BUILD_DISABLE == bcond)
+	{
+	    Sprite gray1 = AGG::GetICN(ICN::BLDGXTRA, 0);
+	    gray1.GrayScale();
+	    gray1.Blit(area.x, area.y);
+	}
 	else
-	    sprite_deny.Blit(dst_pt);
-    }
+	{
+	    AGG::GetICN(ICN::BLDGXTRA, 0).Blit(area.x, area.y);
+	}
 
-    // status bar
-    if(!castle.isBuild(building))
-    {
-	dst_pt.x = area.x;
-	dst_pt.y = area.y + 58;
-	AGG::GetICN(ICN::CASLXTRA, allow_buy ? 1 : 2).Blit(dst_pt);
-    }
+	// build image
+        if(BUILD_DISABLE == bcond && BUILD_TAVERN == building)
+	    // skip tavern necr
+	    Display::Get().FillRect(0, Rect(area.x + 1, area.y + 1, 135, 57));
+	else
+	    AGG::GetICN(ICN::Get4Building(castle.GetRace()), index).Blit(area.x + 1, area.y + 1);
 
-    // name
-    Text text(Castle::GetStringBuilding(building, castle.GetRace()), Font::SMALL);
-    dst_pt.x = area.x + 68 - text.w() / 2;
-    dst_pt.y = area.y + 59;
-    text.Blit(dst_pt);
+	const Sprite & sprite_allow = AGG::GetICN(ICN::TOWNWIND, 11);
+	const Sprite & sprite_deny  = AGG::GetICN(ICN::TOWNWIND, 12);
+	const Sprite & sprite_money = AGG::GetICN(ICN::TOWNWIND, 13);
+
+	Point dst_pt(area.x + 115, area.y + 40);
+
+	// indicator
+	if(bcond == ALREADY_BUILT)
+	    sprite_allow.Blit(dst_pt);
+	else
+	if(bcond == BUILD_DISABLE)
+	{
+	    Sprite gray_deny = sprite_deny;
+	    gray_deny.GrayScale();
+	    gray_deny.Blit(dst_pt);
+	}
+	else
+	if(bcond != ALLOW_BUILD)
+        {
+	    if(LACK_RESOURCES == bcond)
+		sprite_money.Blit(dst_pt);
+	    else
+		sprite_deny.Blit(dst_pt);
+	}
+
+	// status bar
+	if(bcond != BUILD_DISABLE && bcond != ALREADY_BUILT)
+	{
+	    dst_pt.x = area.x;
+	    dst_pt.y = area.y + 58;
+	    AGG::GetICN(ICN::CASLXTRA, bcond == ALLOW_BUILD ? 1 : 2).Blit(dst_pt);
+	}
+
+	// name
+	Text text(Castle::GetStringBuilding(building, castle.GetRace()), Font::SMALL);
+	dst_pt.x = area.x + 68 - text.w() / 2;
+        dst_pt.y = area.y + 59;
+	text.Blit(dst_pt);
+    }
 }
 
 
@@ -431,26 +440,28 @@ const std::string & BuildingInfo::GetDescription(void) const
 
 bool BuildingInfo::QueueEventProcessing(void)
 {
-    if(disable) return false;
-
     LocalEvent & le = LocalEvent::Get();
+
     if(le.MouseClickLeft(area))
     {
-	if(!castle.isBuild(BUILD_CASTLE))
-	    Dialog::Message("", _("For this action it is necessary first to build a castle."), Font::BIG, Dialog::OK);
-	else
-	if(castle.isBuild(building))
+	if(bcond == ALREADY_BUILT)
 	    Dialog::Message(GetName(), GetDescription(), Font::BIG, Dialog::OK);
 	else
+	if(bcond == ALLOW_BUILD || bcond == REQUIRES_BUILD || bcond == LACK_RESOURCES)
 	    return DialogBuyBuilding(true);
+	else
+	    Dialog::Message("", GetConditionDescription(), Font::BIG, Dialog::OK);
     }
     else
     if(le.MousePressRight(area))
     {
-	if(castle.isBuild(building))
+	if(bcond == ALREADY_BUILT)
 	    Dialog::Message(GetName(), GetDescription(), Font::BIG);
 	else
+	if(bcond == ALLOW_BUILD || bcond == REQUIRES_BUILD || bcond == LACK_RESOURCES)
 	    DialogBuyBuilding(false);
+	else
+	    Dialog::Message("", GetConditionDescription(), Font::BIG);
     }
     return false;
 }
@@ -464,32 +475,37 @@ bool BuildingInfo::DialogBuyBuilding(bool buttons) const
     Cursor & cursor = Cursor::Get();
     cursor.Hide();
 
-    TextBox box1(description, Font::BIG, BOXAREA_WIDTH);
+    std::string box1str = description;
+
+    if(ALLOW_BUILD != bcond)
+    {
+	const std::string & ext = GetConditionDescription();
+
+	if(! ext.empty())
+	{
+	    box1str.append("\n \n");
+	    box1str.append(ext);
+	}
+    }
+
+    TextBox box1(box1str, Font::BIG, BOXAREA_WIDTH);
 
     // prepare requires build string
     std::string str;
-    std::bitset<32> requires(castle.GetBuildingRequires(building));
-    if(requires.any())
+    const u32 requires = castle.GetBuildingRequires(building);
+    const char* sep = ", ";
+
+    for(u32 itr = 0x00000001; itr; itr <<= 1)
+        if((requires & itr) && ! castle.isBuild(itr))
     {
-	u8 count = 0;
-	for(u8 pos = 0; pos < requires.size(); ++pos)
-	{
-	    if(requires.test(pos))
-	    {
-		u32 value = 1;
-		value <<= pos;
-
-		++count;
-
-		if(! castle.isBuild(value))
-		{
-		    str += Castle::GetStringBuilding(static_cast<building_t>(value), castle.GetRace());
-		    if(count < requires.count()) str += ", ";
-		}
-	    }
-	}
+	str.append(Castle::GetStringBuilding(itr, castle.GetRace()));
+	str.append(sep);
     }
 
+    // replace end sep
+    if(str.size())
+	str.replace(str.size() - strlen(sep), strlen(sep), ".");
+
     bool requires_true = str.size();
     Text requires_text(_("Requires:"), Font::BIG);
     TextBox box2(str, Font::BIG, BOXAREA_WIDTH);
@@ -578,46 +594,166 @@ bool BuildingInfo::DialogBuyBuilding(bool buttons) const
     return false;
 }
 
+const char* GetBuildConditionDescription(s8 bcond)
+{
+    switch(bcond)
+    {
+	case NOT_TODAY:
+	    return _("Cannot build. Already built here this turn.");
+	    break;
+
+	case NEED_CASTLE:
+	    return _("For this action it is necessary first to build a castle.");
+
+	default: break;
+    }
+
+    return NULL;
+}
+
+std::string BuildingInfo::GetConditionDescription(void) const
+{
+    std::string res;
+
+    switch(bcond)
+    {
+	case NOT_TODAY:
+	case NEED_CASTLE:
+	    res = GetBuildConditionDescription(bcond);
+	    break;
+
+	case BUILD_DISABLE:
+	    if(building == BUILD_SHIPYARD)
+	    {
+		res = _("Cannot build %{name} because castle is too far from water.");
+		String::Replace(res, "%{name}", Castle::GetStringBuilding(BUILD_SHIPYARD, castle.GetRace()));
+	    }
+	    else
+		res = "disable build.";
+	    break;
+
+	case LACK_RESOURCES:
+	    res = _("Cannot afford %{name}");
+    	    String::Replace(res, "%{name}", GetName());
+	    break;
+
+	case ALREADY_BUILT:
+	    res = _("%{name} is already built");
+	    String::Replace(res, "%{name}", GetName());
+	    break;
+
+	case REQUIRES_BUILD:
+    	    res = _("Cannot build %{name}");
+    	    String::Replace(res, "%{name}", GetName());
+	    break;
+
+	case ALLOW_BUILD:
+    	    res = _("Build %{name}");
+    	    String::Replace(res, "%{name}", GetName());
+	    break;
+
+	default: break;
+    }
+
+    return res;
+}
+
 void BuildingInfo::SetStatusMessage(StatusBar & bar) const
 {
     std::string str;
-    const char* name = GetName();
 
-    if(castle.isBuild(building))
+    switch(bcond)
     {
-        str = _("%{name} is already built");
-        String::Replace(str, "%{name}", name);
-    }
-    else
-    {
-        if(!castle.AllowBuild())
-        {
-            str = _("Cannot build. Already built here this turn.");
-        }
-        else
-        if(castle.AllowBuild() && ! castle.GetKingdom().AllowPayment(GetCost(castle.GetRace(), building)))
-        {
-            str = _("Cannot afford %{name}");
-            String::Replace(str, "%{name}", name);
-        }
-        else
-        if(BUILD_SHIPYARD == building && !castle.HaveNearlySea())
-        {
-            str = _("Cannot build %{name} because castle is too far from water.");
-            String::Replace(str, "%{name}", name);
-        }
-        else
-        if(!castle.AllowBuyBuilding(building))
-        {
-            str = _("Cannot build %{name}");
-            String::Replace(str, "%{name}", name);
-        }
-        else
-        {
-            str = _("Build %{name}");
-            String::Replace(str, "%{name}", name);
-        }
+	case NOT_TODAY:
+	case ALREADY_BUILT:
+	case NEED_CASTLE:
+	case BUILD_DISABLE:
+	case LACK_RESOURCES:
+	case REQUIRES_BUILD:
+	case ALLOW_BUILD:
+	    str = GetConditionDescription();
+	    break;
+
+	default:
+	    break;
     }
 
     bar.ShowMessage(str);
 }
+
+DwellingItem::DwellingItem(Castle & castle, u32 dw)
+{
+    type = castle.GetActualDwelling(dw);
+    mons = Monster(castle.GetRace(), dw);
+}
+
+DwellingsBar::DwellingsBar(Castle & cstl, u16 sw, u16 sh, u8 col) : castle(cstl)
+{
+    for(u32 dw = DWELLING_MONSTER1; dw <= DWELLING_MONSTER6; dw <<= 1)
+        content.push_back(DwellingItem(castle, dw));
+
+    SetContent(content);
+
+    backsf.Set(sw, sh);
+    backsf.Fill(backsf.GetColorIndex(col));
+    Cursor::DrawCursor(backsf, 0x70, true);
+    SetItemSize(sw, sh);
+}
+
+void DwellingsBar::RedrawBackground(const Rect & pos, Surface & dstsf)
+{
+    backsf.Blit(pos.x, pos.y, dstsf);
+}
+
+void DwellingsBar::RedrawItem(DwellingItem & dwl, const Rect & pos, Surface & dstsf)
+{
+    const Sprite & mons32 = AGG::GetICN(ICN::MONS32, dwl.mons.GetSpriteIndex());
+    mons32.Blit(pos.x + (pos.w - mons32.w()) / 2, pos.y + (pos.h - 3 - mons32.h()));
+
+    if(castle.isBuild(dwl.type))
+    {
+        // count
+        Text text(GetString(castle.GetDwellingLivedCount(dwl.type)), Font::SMALL);
+        text.Blit(pos.x + pos.w - text.w() - 3, pos.y + pos.h - text.h() - 1);
+
+        u8 grown = dwl.mons.GetGrown();
+        if(castle.isBuild(BUILD_WELL)) grown += Castle::GetGrownWell();
+        if(castle.isBuild(BUILD_WEL2) && DWELLING_MONSTER1 == dwl.type) grown += Castle::GetGrownWel2();
+
+        // grown
+        text.Set("+" + GetString(grown), Font::YELLOW_SMALL);
+        text.Blit(pos.x + pos.w - text.w() - 3, pos.y + 2);
+    }
+    else
+        AGG::GetICN(ICN::CSLMARKER, 0).Blit(pos.x + pos.w - 10, pos.y + 4, dstsf);
+}
+
+bool DwellingsBar::ActionBarSingleClick(const Point & cursor, DwellingItem & dwl, const Rect & pos)
+{
+    if(castle.isBuild(dwl.type))
+    {
+        castle.RecruitMonster(Dialog::RecruitMonster(dwl.mons, castle.GetDwellingLivedCount(dwl.type), true));
+    }
+    else
+    if(!castle.isBuild(BUILD_CASTLE))
+        Dialog::Message("", GetBuildConditionDescription(NEED_CASTLE), Font::BIG, Dialog::OK);
+    else
+    {
+        BuildingInfo dwelling(castle, static_cast<building_t>(dwl.type));
+
+        if(dwelling.DialogBuyBuilding(true))
+        {
+            AGG::PlaySound(M82::BUILDTWN);
+            castle.BuyBuilding(dwl.type);
+        }
+    }
+
+    return true;
+}
+
+bool DwellingsBar::ActionBarPressRight(const Point & cursor, DwellingItem & dwl, const Rect & pos)
+{
+    Dialog::ArmyInfo(Troop(dwl.mons, castle.GetDwellingLivedCount(dwl.type)), 0);
+
+    return true;
+}
