@@ -23,7 +23,6 @@
 #define H2CASTLE_H
 
 #include <vector>
-#include <bitset>
 #include <string>
 #include "mageguild.h"
 #include "color.h"
@@ -69,12 +68,14 @@ enum building_t
     DWELLING_MONSTER4       = 0x00800000,
     DWELLING_MONSTER5       = 0x01000000,
     DWELLING_MONSTER6       = 0x02000000,
+    DWELLING_MONSTERS       = DWELLING_MONSTER1 | DWELLING_MONSTER2 | DWELLING_MONSTER3 | DWELLING_MONSTER4 | DWELLING_MONSTER5 | DWELLING_MONSTER6,
     DWELLING_UPGRADE2       = 0x04000000,
     DWELLING_UPGRADE3       = 0x08000000,
     DWELLING_UPGRADE4       = 0x10000000,
     DWELLING_UPGRADE5       = 0x20000000,
     DWELLING_UPGRADE6       = 0x40000000,
-    DWELLING_UPGRADE7       = 0x80000000        // black dragon
+    DWELLING_UPGRADE7       = 0x80000000,        // black dragon
+    DWELLING_UPGRADES       = DWELLING_UPGRADE2 | DWELLING_UPGRADE3 | DWELLING_UPGRADE4 | DWELLING_UPGRADE5 | DWELLING_UPGRADE6 | DWELLING_UPGRADE7
 };
 
 enum buildcond_t
@@ -82,11 +83,11 @@ enum buildcond_t
     NOT_TODAY		= -1,
     ALREADY_BUILT	= -2,
     NEED_CASTLE		= -3,
-    CASTLE_DISABLE	= -4,
-    SHIPYARD_DISABLE	= -5,
-    UNKNOWN_UPGRADE	= -6,
-    REQUIRES_BUILD	= -7,
-    LACK_RESOURCES	= -8,
+    BUILD_DISABLE	= -4,
+    UNKNOWN_UPGRADE	= -5,
+    REQUIRES_BUILD	= -6,
+    LACK_RESOURCES	= -7,
+    UNKNOWN_COND	= 0,
     ALLOW_BUILD		= 1
 };
 
@@ -138,8 +139,8 @@ public:
     u16 GetDwellingLivedCount(u32) const;
     u32 GetActualDwelling(u32) const;
 
-    bool RecruitMonster(u32 dw, u16 count);
-    u16  RecruitMaxMonster(u32 dw);
+    bool RecruitMonsterFromDwelling(u32 dw, u16 count);
+    bool RecruitMonster(const Troop &);
     void RecruitAllMonster(void);
 
     void ChangeColor(Color::color_t cl);
@@ -164,17 +165,23 @@ public:
 
     bool AllowBuild(void) const{ return Modes(ALLOWBUILD); }
     bool AllowBuyBuilding(u32) const;
-    buildcond_t CheckBuyBuilding(u32) const;
     bool isBuild(u32 bd) const{ return building & bd; }
     bool BuyBuilding(u32);
     bool AllowBuyBoat(void) const;
     bool BuyBoat(void);
     u32 GetBuildingRequires(u32) const;
 
+    s8 CheckBuyBuilding(u32) const;
+    static s8 GetAllBuildingStatus(const Castle &);
+
     void Scoute(void) const;
 
-    static const char* GetStringBuilding(u32, u8 race = 0);
-    static const char* GetDescriptionBuilding(u32, u8 race = 0);
+    std::string GetStringBuilding(u32) const;
+    std::string GetDescriptionBuilding(u32) const;
+
+    static const char* GetStringBuilding(u32, u8 race);
+    static const char* GetDescriptionBuilding(u32, u8 race);
+
     static ICN::icn_t GetICNBuilding(u32, u8);
     static ICN::icn_t GetICNBoat(const u8 & race);
     u32 GetUpgradeBuilding(u32) const;
