@@ -22,7 +22,6 @@
 
 #include <string>
 #include <vector>
-#include <bitset>
 #include "agg.h"
 #include "world.h"
 #include "button.h"
@@ -174,9 +173,13 @@ u32 Castle::OpenTown(void)
 
     // draw castle sprite
     dst_pt.x = cur_pt.x + 460;
-    dst_pt.y = cur_pt.y + 0;
+    dst_pt.y = cur_pt.y + 5;
     DrawImageCastle(dst_pt);
 
+    // castle name
+    Text text(GetName(), Font::SMALL);
+    text.Blit(cur_pt.x + 536 - text.w() / 2, cur_pt.y + 1);
+
     //
     BuildingInfo dwelling1(*this, DWELLING_MONSTER1);
     dwelling1.SetPos(cur_pt.x + 5, cur_pt.y + 2);
@@ -217,7 +220,7 @@ u32 Castle::OpenTown(void)
     buildingMageGuild.Redraw();
 
     // tavern
-    BuildingInfo buildingTavern(*this, (race == Race::NECR ? BUILD_SHRINE : BUILD_TAVERN));
+    BuildingInfo buildingTavern(*this, BUILD_TAVERN);
     buildingTavern.SetPos(cur_pt.x + 149, cur_pt.y + 157);
     buildingTavern.Redraw();
 
@@ -288,7 +291,7 @@ u32 Castle::OpenTown(void)
     SpriteCursor cursorFormat(AGG::GetICN(ICN::HSICONS, 11), army.isSpreadFormat() ? pointSpreadArmyFormat : pointGroupedArmyFormat);
     if(isBuild(BUILD_CAPTAIN))
     {
-	Text text(_("Attack Skill") + std::string(" "), Font::SMALL);
+	text.Set(_("Attack Skill") + std::string(" "), Font::SMALL);
 	dst_pt.x = cur_pt.x + 535;
 	dst_pt.y = cur_pt.y + 168;
 	text.Blit(dst_pt);
@@ -507,7 +510,7 @@ u32 Castle::OpenTown(void)
 	else
 	if(le.MouseCursor(buildingMageGuild.GetArea())) buildingMageGuild.SetStatusMessage(statusBar);
 	else
-	if(le.MouseCursor(buildingTavern.GetArea()) && !buildingTavern.IsDisable()) buildingTavern.SetStatusMessage(statusBar);
+	if(le.MouseCursor(buildingTavern.GetArea())) buildingTavern.SetStatusMessage(statusBar);
 	else
 	if(le.MouseCursor(buildingThievesGuild.GetArea())) buildingThievesGuild.SetStatusMessage(statusBar);
 	else
