@@ -73,7 +73,6 @@ namespace Dialog
 	UPGRADE_DISABLE = MAX,
 
 	READONLY= 0x2000,
-	BATTLE	= 0x4000,
 	BUTTONS = (YES|OK|NO|CANCEL)
     };
 
@@ -95,8 +94,8 @@ namespace Dialog
     void ExtSettings(bool);
 
     // other info
-    u8   LevelUpSelectSkill(const std::string &, const std::string &, const Skill::Secondary &, const Skill::Secondary &);
-    bool SelectGoldOrExp(const std::string &header, const std::string &message, const u16 gold, const u16 expr);
+    u8   LevelUpSelectSkill(const std::string &, const std::string &, const Skill::Secondary &, const Skill::Secondary &, Heroes &);
+    bool SelectGoldOrExp(const std::string &header, const std::string &message, const u16 gold, const u16 expr, const Heroes &);
 
     void SpellInfo(const Spell &, const bool ok_button = true);
     void SpellInfo(const std::string &, const std::string &, const Spell &, const bool ok_button = true);
@@ -114,12 +113,15 @@ namespace Dialog
     bool InputString(const std::string &, std::string &);
 
     // recruit monster
-    u16 RecruitMonster(const Monster & monster, u16 available);
+    Troop RecruitMonster(const Monster &, u16 available, bool);
     void DwellingInfo(const Monster & monster, u16 available);
     bool SetGuardian(Heroes &, Troop &, CapturedObject &, bool readonly);
 
     // army info
-    answer_t ArmyInfo(const Troop & troop, u16 flags);
+    u16 ArmyInfo(const Troop & troop, u16 flags);
+    u16 ArmyJoinWithCost(const Troop &, u32 join, u32 gold, Heroes &);
+    u8  ArmySplitTroop(u8 free_slots, u32 max, u32 &);
+
     // dialog marketplace
     void Marketplace(bool fromTradingPost = false);
     void MakeGiftResource(void);
