@@ -31,7 +31,7 @@
 #include "dialog.h"
 
 void RedrawCurrentInfo(const Point & pos, u16 available, u32 result,
-	    const payment_t & paymentMonster, const payment_t & paymentCosts)
+	    const payment_t & paymentMonster, const payment_t & paymentCosts, const Funds & funds, const std::string & label)
 {
     Text text;
 
@@ -41,20 +41,25 @@ void RedrawCurrentInfo(const Point & pos, u16 available, u32 result,
     text.Blit(pos.x + 70 - text.w() / 2, pos.y + 130);
     text.Set(GetString(result), Font::BIG);
     text.Blit(pos.x + 167 - text.w() / 2, pos.y + 160);
+    const std::string sgold = GetString(paymentCosts.gold) + " " + "(" + GetString(funds.gold - paymentCosts.gold) + ")";
+    u8 rsext = paymentMonster.GetValidItems() & ~Resource::GOLD;
 
-    if(2 == paymentMonster.GetValidItems())
+    if(rsext)
     {
-	text.Set(GetString(paymentCosts.gold), Font::SMALL);
+	text.Set(sgold, Font::SMALL);
 	text.Blit(pos.x + 133 - text.w() / 2, pos.y + 228);
 
-	text.Set(GetString(paymentMonster.GetFirstValidItems(Resource::ALL & ~Resource::GOLD)), Font::SMALL);
+	text.Set(GetString(paymentCosts.Get(rsext)) + " " + "(" + GetString(funds.Get(rsext) - paymentCosts.Get(rsext)) + ")", Font::SMALL);
 	text.Blit(pos.x + 195 - text.w() / 2, pos.y + 228);
     }
     else
     {
-	text.Set(GetString(paymentCosts.gold), Font::SMALL);
+	text.Set(sgold, Font::SMALL);
 	text.Blit(pos.x + 160 - text.w() / 2, pos.y + 228);
     }
+
+    text.Set(label, Font::SMALL);
+    text.Blit(pos.x + 165 - text.w() / 2, pos.y + 180);
 }
 
 void RedrawResourceInfo(const Surface & sres, const Point & pos, s32 value,
@@ -73,39 +78,17 @@ void RedrawResourceInfo(const Surface & sres, const Point & pos, s32 value,
     text.Blit(dst_pt);
 }
 
-u16 Dialog::RecruitMonster(const Monster & monster, u16 available)
+void RedrawStaticInfo(const Rect & pos, const Monster & monster, bool label)
 {
-    Display & display = Display::Get();
-    LocalEvent & le = LocalEvent::Get();
-
-    // cursor
-    Cursor & cursor = Cursor::Get();
-    const Cursor::themes_t oldcursor = cursor.Themes();
-    cursor.Hide();
-    cursor.SetThemes(Cursor::POINTER);
-
-    // calculate max count
-    u32 max = 0;
-    const payment_t paymentMonster = monster.GetCost();
-    const Kingdom & kingdom = world.GetKingdom(Settings::Get().CurrentColor());
-
-    while(kingdom.AllowPayment(paymentMonster * max) && max < available) ++max;
-
-    u32 result = max;
-
-    payment_t paymentCosts(paymentMonster * result);
-
-    const Sprite & box = AGG::GetICN(ICN::RECRBKG, 0);
-    const Rect pos((display.w() - box.w()) / 2, Settings::Get().QVGA() ? (display.h() - box.h()) / 2 - 15 : (display.h() - box.h()) / 2, box.w(), box.h());
-
-    Background back(pos);
-    back.Save();
-
-    box.Blit(pos.x, pos.y);
-
+    Text text;
     Point dst_pt;
     std::string str;
-    Text text;
+
+    const Sprite & box = AGG::GetICN(ICN::RECRBKG, 0);
+    box.Blit(pos.x, pos.y);
+
+    payment_t paymentMonster = monster.GetCost();
+    bool extres = 2 == paymentMonster.GetValidItemsCount();
 
     // smear hardcore text "Cost per troop:"
     const Sprite & smear = AGG::GetICN(ICN::TOWNNAME, 0);
@@ -132,7 +115,12 @@ u16 Dialog::RecruitMonster(const Monster & monster, u16 available)
     dst_pt.y = pos.y + 130 - smon.h();
     smon.Blit(dst_pt);
 
-    bool extres = 2 == paymentMonster.GetValidItems();
+    // change label
+    if(label)
+    {
+	text.Set("( change )", Font::YELLOW_SMALL);
+	text.Blit(pos.x + 68 - text.w() / 2, pos.y + 80);
+    }
 
     // info resource
     // gold
@@ -222,12 +210,70 @@ u16 Dialog::RecruitMonster(const Monster & monster, u16 available)
     dst_pt.y = pos.y + 163;
     text.Blit(dst_pt);
 
-    Background static_info(Rect(pos.x + 16, pos.y + 125, pos.w - 32, 122));
-    static_info.Save();
+}
 
-    RedrawCurrentInfo(pos, available, result, paymentMonster, paymentCosts);
+const char* SwitchMaxMinButtons(Button & btnMax, Button & btnMin, bool max)
+{
+    if(btnMax.isEnable() || btnMin.isEnable())
+    {
+	if(max)
+	{
+	    btnMax.SetDisable(true);
+	    btnMin.SetDisable(false);
+	}
+	else
+	{
+	    btnMin.SetDisable(true);
+	    btnMax.SetDisable(false);
+	}
+
+	return max ? "max" : "min";
+    }
+
+    return "";
+}
+
+u32 CalculateMax(const Monster & monster, const Kingdom & kingdom, u32 available)
+{
+    u32 max = 0;
+    while(kingdom.AllowPayment(monster.GetCost() * (max + 1)) && (max + 1) <= available) ++max;
+
+    return max;
+}
+
+Troop Dialog::RecruitMonster(const Monster & monster0, u16 available, bool ext)
+{
+    Display & display = Display::Get();
+    LocalEvent & le = LocalEvent::Get();
+
+    // cursor
+    Cursor & cursor = Cursor::Get();
+    const Cursor::themes_t oldcursor = cursor.Themes();
+    cursor.Hide();
+    cursor.SetThemes(Cursor::POINTER);
+
+    // calculate max count
+    Monster monster = monster0;
+    payment_t paymentMonster = monster.GetCost();
+    const Kingdom & kingdom = world.GetKingdom(Settings::Get().CurrentColor());
+
+    u32 max = CalculateMax(monster, kingdom, available);
+    u32 result = max;
+
+    payment_t paymentCosts(paymentMonster * result);
+
+    const Sprite & box = AGG::GetICN(ICN::RECRBKG, 0);
+    const Rect pos((display.w() - box.w()) / 2, Settings::Get().QVGA() ? (display.h() - box.h()) / 2 - 15 : 0, box.w(), box.h());
+
+    Background back(pos);
+    back.Save();
+
+    const Rect rtChange(pos.x + 25, pos.y + 35, 85, 95);
+    RedrawStaticInfo(pos, monster, ext && monster0.GetDowngrade() != monster0);
 
     // buttons
+    Point dst_pt;
+
     dst_pt.x = pos.x + 34;
     dst_pt.y = pos.y + 249;
     Button buttonOk(dst_pt, ICN::RECRUIT, 8, 9);
@@ -239,6 +285,8 @@ u16 Dialog::RecruitMonster(const Monster & monster, u16 available)
     dst_pt.x = pos.x + 230;
     dst_pt.y = pos.y + 155;
     Button buttonMax(dst_pt, ICN::RECRUIT, 4, 5);
+    Button buttonMin(dst_pt, ICN::BTNMIN, 0, 1);
+
     dst_pt.x = pos.x + 208;
     dst_pt.y = pos.y + 156;
     Button buttonUp(dst_pt, ICN::RECRUIT, 0, 1);
@@ -247,9 +295,27 @@ u16 Dialog::RecruitMonster(const Monster & monster, u16 available)
     dst_pt.y = pos.y + 171;
     Button buttonDn(dst_pt, ICN::RECRUIT, 2, 3);
 
+    const Rect rtWheel(pos.x + 130, pos.y +155, 100, 30);
+
+    if(0 == result)
+    {
+	buttonOk.Press();
+	buttonOk.SetDisable(true);
+	buttonMax.Press();
+	buttonMin.Press();
+	buttonMax.SetDisable(true);
+	buttonMin.SetDisable(true);
+	buttonMax.Draw();
+    }
+
+    const Funds & funds = kingdom.GetFunds();
+    std::string maxmin = SwitchMaxMinButtons(buttonMax, buttonMin, true);
+    RedrawCurrentInfo(pos, available, result, paymentMonster, paymentCosts, funds, maxmin);
+
     buttonOk.Draw();
     buttonCancel.Draw();
-    buttonMax.Draw();
+    if(buttonMax.isEnable()) buttonMax.Draw();
+    if(buttonMin.isEnable()) buttonMin.Draw();
     buttonUp.Draw();
     buttonDn.Draw();
 
@@ -261,44 +327,135 @@ u16 Dialog::RecruitMonster(const Monster & monster, u16 available)
     // str loop
     while(le.HandleEvents())
     {
-	le.MousePressLeft(buttonOk) ? buttonOk.PressDraw() : buttonOk.ReleaseDraw();
+	if(buttonOk.isEnable())
+	    le.MousePressLeft(buttonOk) ? buttonOk.PressDraw() : buttonOk.ReleaseDraw();
 	le.MousePressLeft(buttonCancel) ? buttonCancel.PressDraw() : buttonCancel.ReleaseDraw();
-	le.MousePressLeft(buttonMax) ? buttonMax.PressDraw() : buttonMax.ReleaseDraw();
 	le.MousePressLeft(buttonUp) ? buttonUp.PressDraw() : buttonUp.ReleaseDraw();
 	le.MousePressLeft(buttonDn) ? buttonDn.PressDraw() : buttonDn.ReleaseDraw();
 
+	if(buttonMax.isEnable())
+	    le.MousePressLeft(buttonMax) ? buttonMax.PressDraw() : buttonMax.ReleaseDraw();
+	if(buttonMin.isEnable())
+	    le.MousePressLeft(buttonMin) ? buttonMin.PressDraw() : buttonMin.ReleaseDraw();
+
+	if(ext && le.MouseClickLeft(rtChange))
+	{
+	    if(monster != monster.GetDowngrade())
+	    {
+		monster = monster.GetDowngrade();
+		max = CalculateMax(monster, kingdom, available);
+		result = max;
+		paymentMonster = monster.GetCost();
+		paymentCosts = paymentMonster * result;
+		redraw = true;
+	    }
+	    else
+	    if(monster != monster0)
+	    {
+		monster = monster0;
+		max = CalculateMax(monster, kingdom, available);
+		result = max;
+		paymentMonster = monster.GetCost();
+		paymentCosts = paymentMonster * result;
+		redraw = true;
+	    }
+
+	    if(result == max)
+	    {
+		maxmin = SwitchMaxMinButtons(buttonMax, buttonMin, true);
+	    }
+	}
+
 	if(PressIntKey(0, max, result))
 	{
 	    paymentCosts = paymentMonster * result;
 	    redraw = true;
+	    maxmin.clear();
+
+	    if(result == max)
+	    {
+		maxmin = SwitchMaxMinButtons(buttonMax, buttonMin, true);
+	    }
+	    else
+	    if(result == 1)
+	    {
+		maxmin = SwitchMaxMinButtons(buttonMax, buttonMin, false);
+	    }
 	}
 
-	if(le.MouseClickLeft(buttonUp) && result < max)
+	if((le.MouseWheelUp(rtWheel) || le.MouseClickLeft(buttonUp)) && result < max)
 	{
 	    ++result;
 	    paymentCosts += paymentMonster;
 	    redraw = true;
+	    maxmin.clear();
+
+	    if(result == max)
+	    {
+		maxmin = SwitchMaxMinButtons(buttonMax, buttonMin, true);
+	    }
+	    else
+	    if(result == 1)
+	    {
+		maxmin = SwitchMaxMinButtons(buttonMax, buttonMin, false);
+	    }
 	}
 	else
-	if(le.MouseClickLeft(buttonDn) && result)
+	if((le.MouseWheelDn(rtWheel) || le.MouseClickLeft(buttonDn)) && result)
 	{
 	    --result;
 	    paymentCosts -= paymentMonster;
 	    redraw = true;
+	    maxmin.clear();
+
+	    if(result == max)
+	    {
+		maxmin = SwitchMaxMinButtons(buttonMax, buttonMin, true);
+	    }
+	    else
+	    if(result == 1)
+	    {
+		maxmin = SwitchMaxMinButtons(buttonMax, buttonMin, false);
+	    }
 	}
 	else
-	if(le.MouseClickLeft(buttonMax) && result != max)
+	if(buttonMax.isEnable() && le.MouseClickLeft(buttonMax) && result != max)
 	{
+	    maxmin = SwitchMaxMinButtons(buttonMax, buttonMin, true);
 	    result = max;
 	    paymentCosts = paymentMonster * max;
 	    redraw = true;
 	}
+	else
+	if(buttonMin.isEnable() && le.MouseClickLeft(buttonMin) && result != 1)
+	{
+	    maxmin = SwitchMaxMinButtons(buttonMax, buttonMin, false);
+	    result = 1;
+	    paymentCosts = paymentMonster;
+	    redraw = true;
+	}
 
 	if(redraw)
 	{
 	    cursor.Hide();
-	    static_info.Restore();
-	    RedrawCurrentInfo(pos, available, result, paymentMonster, paymentCosts);
+	    RedrawStaticInfo(pos, monster, ext && monster0.GetDowngrade() != monster0);
+	    RedrawCurrentInfo(pos, available, result, paymentMonster, paymentCosts, funds, maxmin);
+
+	    if(0 == result)
+	    {
+		buttonOk.Press();
+		buttonOk.SetDisable(true);
+		buttonOk.Draw();
+	    }
+	    else
+	    {
+		buttonOk.Release();
+		buttonOk.SetDisable(false);
+		buttonOk.Draw();
+	    }
+
+	    if(buttonMax.isEnable()) buttonMax.Draw();
+	    if(buttonMin.isEnable()) buttonMin.Draw();
 	    cursor.Show();
 	    display.Flip();
 	    redraw = false;
@@ -317,7 +474,7 @@ u16 Dialog::RecruitMonster(const Monster & monster, u16 available)
     cursor.Show();
     display.Flip();
 
-    return result;
+    return Troop(monster, result);
 }
 
 void Dialog::DwellingInfo(const Monster & monster, u16 available)
@@ -357,7 +514,7 @@ void Dialog::DwellingInfo(const Monster & monster, u16 available)
     dst_pt.y = pos.y + 120 - smon.h();
     smon.Blit(dst_pt);
 
-    bool extres = 2 == paymentMonster.GetValidItems();
+    bool extres = 2 == paymentMonster.GetValidItemsCount();
 
     // info resource
     // gold
