@@ -542,38 +542,22 @@ Cursor::themes_t Game::GetCursorFocusHeroes(const Heroes & from_hero, const Maps
 	    break;
 
 	case MP2::OBJN_CASTLE:
-    	{
-    	    const Castle* castle = world.GetCastle(tile.GetIndex());
-    	    if(NULL != castle)
-	    {
-		if(from_hero.GetColor() == castle->GetColor())
-		    return Cursor::CASTLE;
-		else
-		if(from_hero.Modes(Heroes::GUARDIAN))
-		    return Cursor::POINTER;
-		else
-		if(from_hero.isFriends(castle->GetColor()))
-		    return conf.ExtUnionsAllowCastleVisiting() ? Cursor::ACTION : Cursor::POINTER;
-		else
-		if(castle->GetActualArmy().isValid())
-		    return Cursor::DistanceThemes(Cursor::FIGHT, from_hero.GetRangeRouteDays(castle->GetIndex()));
-		else
-		    return Cursor::DistanceThemes(Cursor::ACTION, from_hero.GetRangeRouteDays(castle->GetIndex()));
-	    }
-    	}
-    	break;
-
     	case MP2::OBJ_CASTLE:
     	{
     	    const Castle* castle = world.GetCastle(tile.GetIndex());
 
     	    if(NULL != castle)
 	    {
-		if(from_hero.Modes(Heroes::GUARDIAN))
+		if(tile.GetObject() == MP2::OBJN_CASTLE &&
+		    from_hero.GetColor() == castle->GetColor())
+		    return Cursor::CASTLE;
+		else
+		if(from_hero.Modes(Heroes::GUARDIAN) ||
+		    from_hero.GetIndex() == castle->GetIndex())
 		    return from_hero.GetColor() == castle->GetColor() ? Cursor::CASTLE : Cursor::POINTER;
 		else
 		if(from_hero.GetColor() == castle->GetColor())
-		    return Cursor::DistanceThemes(Cursor::ACTION, from_hero.GetRangeRouteDays(tile.GetIndex()));
+		    return Cursor::DistanceThemes(Cursor::ACTION, from_hero.GetRangeRouteDays(castle->GetIndex()));
 		else
 		if(from_hero.isFriends(castle->GetColor()))
 		    return conf.ExtUnionsAllowCastleVisiting() ? Cursor::ACTION : Cursor::POINTER;
@@ -1120,6 +1104,14 @@ void Game::MouseCursorAreaClickLeft(s32 index_maps)
     Heroes* from_hero = GameFocus::GetHeroes();
     const Maps::Tiles & tile = world.GetTiles(index_maps);
 
+    // correct index for castle
+    if(MP2::OBJN_CASTLE == tile.GetObject() ||
+	MP2::OBJ_CASTLE == tile.GetObject())
+    {
+    	to_castle = world.GetCastle(index_maps);
+	if(to_castle) index_maps = to_castle->GetIndex();
+    }
+
     switch(Cursor::WithoutDistanceThemes(Cursor::Get().Themes()))
     {
 	case Cursor::HEROES:
@@ -1139,7 +1131,7 @@ void Game::MouseCursorAreaClickLeft(s32 index_maps)
 
         case Cursor::CASTLE:
 	    // focus change/open castle
-	    if(NULL != (to_castle = world.GetCastle(index_maps)))
+	    if(to_castle)
 	    {
 		Castle* from_castle = GameFocus::GetCastle();
 
@@ -1155,12 +1147,6 @@ void Game::MouseCursorAreaClickLeft(s32 index_maps)
 	    break;
 
         case Cursor::FIGHT:
-	    if(NULL != (to_castle = world.GetCastle(index_maps)))
-		ShowPathOrStartMoveHero(from_hero, to_castle->GetIndex());
-	    else
-		ShowPathOrStartMoveHero(from_hero, index_maps);
-	    break;
-
         case Cursor::MOVE:
         case Cursor::BOAT:
         case Cursor::ANCHOR:
@@ -1349,6 +1335,8 @@ void Game::EventFileDialog(Game::menu_t & ret)
 
 void Game::EventSystemDialog(void)
 {
+    const Settings & conf = Settings::Get();
+
     // Change and save system settings
     const u8 changes = Dialog::SystemOptions();
     Interface::Basic & I = Interface::Basic::Get();
@@ -1362,12 +1350,24 @@ void Game::EventSystemDialog(void)
 	    I.gameArea.SetCenter(GameFocus::GetCenter());
         I.SetRedraw(REDRAW_GAMEAREA);
 
-	if(Settings::Get().ExtGameHideInterface())
+	if(conf.ExtGameHideInterface())
 	    I.controlPanel.ResetTheme();
     }
 
+    // interface themes
     if(0x08 & changes)
+    {
         I.SetRedraw(REDRAW_ICONS | REDRAW_BUTTONS | REDRAW_STATUS | REDRAW_BORDER);
+    }
+
+    // interface hide/show
+    if(0x04 & changes)
+    {
+	I.SetHideInterface(conf.ExtGameHideInterface());
+        I.SetRedraw(REDRAW_ALL);
+	I.Redraw();
+	GameFocus::Reset(GameFocus::HEROES);
+    }
 }
 
 void Game::EventExit(menu_t & ret)
@@ -1572,7 +1572,7 @@ void Game::ShowEventDay(void)
 	it = events.begin(); it != events.end(); ++it)
     {
     	AGG::PlayMusic(MUS::NEWS, false);
-	if((*it).resource.GetValidItems())
+	if((*it).resource.GetValidItemsCount())
 	    Dialog::ResourceInfo("", (*it).message, (*it).resource);
 	else
 	if((*it).message.size())
@@ -1720,3 +1720,12 @@ void Game::EventSwitchShowControlPanel(void)
 	Interface::Basic::Get().SetRedraw(REDRAW_GAMEAREA);
     }
 }
+
+void Game::EventDebug1(void)
+{
+    AGG::Cache::Get().Dump();
+}
+
+void Game::EventDebug2(void)
+{
+}
