@@ -44,6 +44,33 @@ struct ComparsionDistance
     s32 center;
 };
 
+Maps::IndexesDistance::IndexesDistance(const s32 & from, const s32 & center, u16 dist, u8 sort)
+{
+    Assign(from, GetAroundIndexes(center, dist, sort), sort);
+}
+
+Maps::IndexesDistance::IndexesDistance(const s32 & from, const Indexes & indexes, u8 sort)
+{
+    Assign(from, indexes, sort);
+}
+
+void Maps::IndexesDistance::Assign(const s32 & from, const Indexes & indexes, u8 sort)
+{
+    reserve(indexes.size());
+
+    for(Indexes::const_iterator
+        it = indexes.begin(); it != indexes.end(); ++it)
+        push_back(IndexDistance(*it, Maps::GetApproximateDistance(from, *it)));
+
+    if(sort)
+    {
+        if(1 == sort)
+            std::sort(begin(), end(), IndexDistance::Shortest);
+        else
+            std::sort(begin(), end(), IndexDistance::Longest);
+    }
+}
+
 bool TileIsObject(s32 index, u8 obj)
 {
     return obj == world.GetTiles(index).GetObject();
@@ -59,7 +86,7 @@ bool TileIsObjects(s32 index, const u8* objs)
     return false;
 }
 
-MapsIndexes & MapsIndexesFilteredObjects(MapsIndexes & indexes, const u8* objs)
+Maps::Indexes & MapsIndexesFilteredObjects(Maps::Indexes & indexes, const u8* objs)
 {
     indexes.resize(std::distance(indexes.begin(),
 	    std::remove_if(indexes.begin(), indexes.end(), std::not1(std::bind2nd(std::ptr_fun(&TileIsObjects), objs)))));
@@ -67,7 +94,7 @@ MapsIndexes & MapsIndexesFilteredObjects(MapsIndexes & indexes, const u8* objs)
     return indexes;
 }
 
-MapsIndexes & MapsIndexesFilteredObject(MapsIndexes & indexes, u8 obj)
+Maps::Indexes & MapsIndexesFilteredObject(Maps::Indexes & indexes, u8 obj)
 {
     indexes.resize(std::distance(indexes.begin(),
 	    std::remove_if(indexes.begin(), indexes.end(), std::not1(std::bind2nd(std::ptr_fun(&TileIsObject), obj)))));
@@ -153,6 +180,11 @@ bool Maps::isValidDirection(const s32 & from, u16 vector)
     return false;
 }
 
+Point Maps::GetPoint(const s32 & index)
+{
+    return Point(index % world.w(), index / world.h());
+}
+
 bool Maps::isValidAbsPoint(const Point & pt)
 {
     return isValidAbsPoint(pt.x, pt.y);
@@ -187,41 +219,41 @@ s32 Maps::GetIndexFromAbsPoint(s16 px, s16 py)
     return res;
 }
 
-MapsIndexes Maps::GetAllIndexes(void)
+Maps::Indexes Maps::GetAllIndexes(void)
 {
-    MapsIndexes result(world.w() * world.h());
-    for(MapsIndexes::iterator
+    Indexes result;
+    result.assign(world.w() * world.h(), 0);
+
+    for(Indexes::iterator
 	it = result.begin(); it != result.end(); ++it)
 	*it = std::distance(result.begin(), it);
     return result;
 }
 
-MapsIndexes Maps::GetAroundIndexes(const s32 & center, u16 filter)
+Maps::Indexes Maps::GetAroundIndexes(const s32 & center)
 {
-    MapsIndexes result;
+    Indexes result;
     result.reserve(8);
 
     if(isValidAbsIndex(center))
 	for(Direction::vector_t direct = Direction::TOP_LEFT; direct != Direction::CENTER; ++direct)
-	    if((filter & direct) && 
-	    isValidDirection(center, direct))
+	    if(isValidDirection(center, direct))
 		result.push_back(GetDirectionIndex(center, direct));
 
     return result;
 }
 
-MapsIndexes Maps::GetDistanceIndexes(const s32 & center, u16 dist, bool sort)
+Maps::Indexes Maps::GetAroundIndexes(const s32 & center, u16 dist, bool sort)
 {
-    MapsIndexes results;
+    Indexes results;
     results.reserve(dist * 12);
 
-    const s16 cx = center % world.w();
-    const s16 cy = center / world.w();
+    const Point cp = GetPoint(center);
 
-    for(s16 xx = cx - dist; xx <= cx + dist; ++xx)
-	for(s16 yy = cy - dist; yy <= cy + dist; ++yy)
+    for(s16 xx = cp.x - dist; xx <= cp.x + dist; ++xx)
+	for(s16 yy = cp.y - dist; yy <= cp.y + dist; ++yy)
     {
-	if(isValidAbsPoint(xx, yy) && (xx != cx || yy != cy))
+	if(isValidAbsPoint(xx, yy) && (xx != cp.x || yy != cp.y))
 	    results.push_back(GetIndexFromAbsPoint(xx, yy));
     }
 
@@ -231,11 +263,37 @@ MapsIndexes Maps::GetDistanceIndexes(const s32 & center, u16 dist, bool sort)
     return results;
 }
 
+Maps::Indexes Maps::GetDistanceIndexes(const s32 & center, u16 dist)
+{
+    Indexes results;
+    results.reserve(dist * 6);
+
+    const Point cp = GetPoint(center);
+
+    for(s16 xx = cp.x - dist; xx <= cp.x + dist; ++xx)
+    {
+       if(isValidAbsPoint(xx, cp.y - dist))
+           results.push_back(GetIndexFromAbsPoint(xx, cp.y - dist));
+       if(isValidAbsPoint(xx, cp.y + dist))
+           results.push_back(GetIndexFromAbsPoint(xx, cp.y + dist));
+    }
+
+    for(s16 yy = cp.y - dist + 1; yy < cp.y + dist; ++yy)
+    {
+       if(isValidAbsPoint(cp.x - dist, yy))
+           results.push_back(GetIndexFromAbsPoint(cp.x - dist, yy));
+       if(isValidAbsPoint(cp.x + dist, yy))
+           results.push_back(GetIndexFromAbsPoint(cp.x + dist, yy));
+    }
+
+    return results;
+}
+
 void Maps::ClearFog(const s32 & index, u8 scoute, u8 color)
 {
     if(0 != scoute && isValidAbsIndex(index))
     {
-	const Point center(index % world.w(), index / world.w());
+	const Point center = GetPoint(index);
 	const Settings & conf = Settings::Get();
 
 	// AI advantage
@@ -261,39 +319,39 @@ void Maps::ClearFog(const s32 & index, u8 scoute, u8 color)
     }
 }
 
-MapsIndexes Maps::ScanAroundObjects(const s32 & center, const u8* objs)
+Maps::Indexes Maps::ScanAroundObjects(const s32 & center, const u8* objs)
 {
-    MapsIndexes results = Maps::GetAroundIndexes(center);
+    Indexes results = Maps::GetAroundIndexes(center);
     return MapsIndexesFilteredObjects(results, objs);
 }
 
-MapsIndexes Maps::ScanAroundObject(const s32 & center, u8 obj)
+Maps::Indexes Maps::ScanAroundObject(const s32 & center, u8 obj)
 {
-    MapsIndexes results = Maps::GetAroundIndexes(center);
+    Maps::Indexes results = Maps::GetAroundIndexes(center);
     return MapsIndexesFilteredObject(results, obj);
 }
 
-MapsIndexes Maps::ScanDistanceObject(const s32 & center, u8 obj, u16 dist)
+Maps::Indexes Maps::ScanAroundObject(const s32 & center, u16 dist, u8 obj)
 {
-    MapsIndexes results = Maps::GetDistanceIndexes(center, dist, true);
+    Indexes results = Maps::GetAroundIndexes(center, dist, true);
     return MapsIndexesFilteredObject(results, obj);
 }
 
-MapsIndexes Maps::ScanDistanceObjects(const s32 & center, const u8* objs, u16 dist)
+Maps::Indexes Maps::ScanAroundObjects(const s32 & center, u16 dist, const u8* objs)
 {
-    MapsIndexes results = Maps::GetDistanceIndexes(center, dist, true);
+    Indexes results = Maps::GetAroundIndexes(center, dist, true);
     return MapsIndexesFilteredObjects(results, objs);
 }
 
-MapsIndexes Maps::GetObjectPositions(u8 obj, bool check_hero)
+Maps::Indexes Maps::GetObjectPositions(u8 obj, bool check_hero)
 {
-    MapsIndexes results = GetAllIndexes();
+    Maps::Indexes results = GetAllIndexes();
     MapsIndexesFilteredObject(results, obj);
 
     if(check_hero && obj != MP2::OBJ_HEROES)
     {
-	const MapsIndexes & v = GetObjectPositions(MP2::OBJ_HEROES, false);
-	for(MapsIndexes::const_iterator
+	const Indexes & v = GetObjectPositions(MP2::OBJ_HEROES, false);
+	for(Indexes::const_iterator
 	    it = v.begin(); it != v.end(); ++it)
 	{
 	    const Heroes* hero = world.GetHeroes(*it);
@@ -305,16 +363,16 @@ MapsIndexes Maps::GetObjectPositions(u8 obj, bool check_hero)
     return results;
 }
 
-MapsIndexes Maps::GetObjectPositions(const s32 & center, u8 obj, bool check_hero)
+Maps::Indexes Maps::GetObjectPositions(const s32 & center, u8 obj, bool check_hero)
 {
-    MapsIndexes results = Maps::GetObjectPositions(obj, check_hero);
+    Indexes results = Maps::GetObjectPositions(obj, check_hero);
     std::sort(results.begin(), results.end(), ComparsionDistance(center));
     return results;
 }
 
-MapsIndexes Maps::GetObjectsPositions(const u8* objs)
+Maps::Indexes Maps::GetObjectsPositions(const u8* objs)
 {
-    MapsIndexes results = GetAllIndexes();
+    Indexes results = GetAllIndexes();
     return MapsIndexesFilteredObjects(results, objs);
 }
 
@@ -347,15 +405,20 @@ bool MapsTileIsUnderProtection(const s32 from, const s32 index) /* from: center,
     return result;
 }
 
+bool Maps::IsNearTiles(const s32 & index1, const s32 & index2)
+{
+    return DIRECTION_ALL & Direction::Get(index1, index2);
+}
+
 bool Maps::TileIsUnderProtection(const s32 & center)
 {
     return MP2::OBJ_MONSTER == world.GetTiles(center).GetObject() ? true :
 	    GetTilesUnderProtection(center).size();
 }
 
-MapsIndexes Maps::GetTilesUnderProtection(const s32 & center)
+Maps::Indexes Maps::GetTilesUnderProtection(const s32 & center)
 {
-    MapsIndexes indexes = Maps::ScanAroundObject(center, MP2::OBJ_MONSTER);
+    Indexes indexes = Maps::ScanAroundObject(center, MP2::OBJ_MONSTER);
 
     indexes.resize(std::distance(indexes.begin(),
 	    std::remove_if(indexes.begin(), indexes.end(),
@@ -369,11 +432,10 @@ MapsIndexes Maps::GetTilesUnderProtection(const s32 & center)
 
 u16 Maps::GetApproximateDistance(const s32 & index1, const s32 & index2)
 {
-    return std::max(std::abs((index1 % world.w()) - (index2 % world.w())), 
-	            std::abs((index1 / world.w()) - (index2 / world.w())));
+    const Size sz(GetPoint(index1) - GetPoint(index2));
+    return std::max(sz.w, sz.h);
 }
 
-
 void Maps::MinimizeAreaForCastle(const Point & center)
 {
     // reset castle ID
@@ -415,7 +477,7 @@ castle size: T and B - sprite, S - shadow, XX - center
       S3S3B1B1XXB1B1
         S4B2B2  B2B2
 */
-    MapsIndexes coords;
+    Indexes coords;
     coords.reserve(21);
 
     // T0
@@ -460,7 +522,7 @@ castle size: T and B - sprite, S - shadow, XX - center
     }
 
     // modify all rnd sprites
-    for(MapsIndexes::const_iterator
+    for(Indexes::const_iterator
 	it = coords.begin(); it != coords.end(); ++it)
     if(isValidAbsIndex(*it))
     {
@@ -486,7 +548,7 @@ castle size: T and B - sprite, S - shadow, XX - center
 void Maps::UpdateSpritesFromTownToCastle(const Point & center)
 {
     // correct area maps sprites
-    MapsIndexes coords;
+    Indexes coords;
     coords.reserve(15);
 
     // T1
@@ -509,7 +571,7 @@ void Maps::UpdateSpritesFromTownToCastle(const Point & center)
     coords.push_back(GetIndexFromAbsPoint(center.x + 2, center.y));
 
     // modify all town sprites
-    for(MapsIndexes::const_iterator
+    for(Indexes::const_iterator
 	it = coords.begin(); it != coords.end(); ++it)
     if(isValidAbsIndex(*it))
     {
@@ -526,12 +588,11 @@ void Maps::UpdateSpritesFromTownToCastle(const Point & center)
     }
 }
 
-u16  Maps::TileIsCoast(const s32 & index, u16 direct)
+u16  Maps::TileIsCoast(const s32 & center, u16 filter)
 {
     u16 result = 0;
-    const MapsIndexes & v = GetAroundIndexes(index, direct);
-    for(MapsIndexes::const_iterator
-        it = v.begin(); it != v.end(); ++it)
-        if(world.GetTiles(*it).isWater()) result |= Direction::Get(index, *it);
+    for(Direction::vector_t direct = Direction::TOP_LEFT; direct != Direction::CENTER; ++direct)
+    if((direct & filter) && isValidDirection(center, direct) &&
+        world.GetTiles(GetDirectionIndex(center, direct)).isWater()) result |= direct;
     return result;
 }
