@@ -26,8 +26,7 @@
 #include "button.h"
 #include "heroes.h"
 #include "heroes_indicator.h"
-#include "selectarmybar.h"
-#include "selectartifactbar.h"
+#include "army_bar.h"
 #include "world.h"
 #include "race.h"
 #include "kingdom.h"
@@ -87,70 +86,37 @@ Dialog::answer_t PocketPC::HeroesOpenDialog(Heroes & hero, bool readonly)
     luckIndicator.Redraw();
 
     // prim skill
-    const Rect ras(dst_rt.x + 74, dst_rt.y + 14, 34, 34);
-    backSprite.Blit(Rect(216, 51, ras.w, ras.h),  ras);
-    text.Set(GetString(hero.GetAttack()));
-    text.Blit(dst_rt.x + 74 + (34 - text.w()) / 2, dst_rt.y + 47);
-
-    const Rect rds(dst_rt.x + 107, dst_rt.y + 14, 34, 34);
-    backSprite.Blit(Rect(216, 84, rds.w, rds.h),  rds);
-    text.Set(GetString(hero.GetDefense()));
-    text.Blit(dst_rt.x + 107 + (34 - text.w()) / 2, dst_rt.y + 47);
-
-    const Rect rps(dst_rt.x + 140, dst_rt.y + 14, 34, 34);
-    backSprite.Blit(Rect(216, 117, rps.w, rps.h), rps);
-    text.Set(GetString(hero.GetPower()));
-    text.Blit(dst_rt.x + 140 + (34 - text.w()) / 2, dst_rt.y + 47);
-
-    const Rect rks(dst_rt.x + 173, dst_rt.y + 14, 34, 34);
-    backSprite.Blit(Rect(216, 150, rks.w, rks.h), rks);
-    text.Set(GetString(hero.GetKnowledge()));
-    text.Blit(dst_rt.x + 173 + (34 - text.w()) / 2, dst_rt.y + 47);
+    PrimarySkillsBar primskill_bar(&hero, true);
+    primskill_bar.SetColRows(4, 1);
+    primskill_bar.SetHSpace(-1);
+    primskill_bar.SetTextOff(0, -1);
+    primskill_bar.SetPos(dst_rt.x + 74, dst_rt.y + 14);
+    primskill_bar.Redraw();
 
     // sec skill
     backSprite.Blit(Rect(21, 198, 267, 36), dst_rt.x + 7, dst_rt.y + 57);
     // secondary skill
-    SecondarySkillBar secskill_bar;
-    secskill_bar.SetPos(dst_rt.x + 9, dst_rt.y + 59);
-    secskill_bar.SetUseMiniSprite();
-    secskill_bar.SetInterval(1);
-    secskill_bar.SetSkills(hero.GetSecondarySkills());
+    SecondarySkillsBar secskill_bar;
+    secskill_bar.SetColRows(8, 1);
+    secskill_bar.SetHSpace(-1);
+    secskill_bar.SetContent(hero.GetSecondarySkills());
+    secskill_bar.SetPos(dst_rt.x + 8, dst_rt.y + 58);
     secskill_bar.Redraw();
 
     // army bar
-    const Rect rt1(36, 267, 43, 53);
-    Surface sfb1(rt1.w, rt1.h);
-    backSprite.Blit(rt1, 0, 0, sfb1);
-    Surface sfc1(rt1.w, rt1.h - 10);
-    Cursor::DrawCursor(sfc1, 0xd6, true);
-
-    SelectArmyBar selectArmy;
-    selectArmy.SetArmy(hero.GetArmy());
-    selectArmy.SetPos(dst_rt.x + 50, dst_rt.y + 170);
-    selectArmy.SetInterval(2);
-    selectArmy.SetBackgroundSprite(sfb1);
-    selectArmy.SetCursorSprite(sfc1);
-    selectArmy.SetUseMons32Sprite();
-    selectArmy.SetSaveLastTroop();
-    if(readonly) selectArmy.SetReadOnly();
-    const Castle* castle = hero.inCastle();
-    if(castle) selectArmy.SetCastle(*castle);
+    ArmyBar selectArmy(&hero.GetArmy(), true, readonly);
+    selectArmy.SetColRows(5, 1);
+    selectArmy.SetPos(dst_rt.x + 51, dst_rt.y + 170);
+    selectArmy.SetHSpace(-1);
     selectArmy.Redraw();
             
     // art bar
-    const Rect rt2(23, 347, 34, 34);
-    Surface sfb2(rt2.w, rt2.h);
-    backSprite.Blit(rt2, 0, 0, sfb2);
-    Surface sfc2(rt2.w, rt2.h);
-    Cursor::DrawCursor(sfc2, 0xd6, true);
-
-    SelectArtifactsBar selectArtifacts(hero);
+    ArtifactsBar selectArtifacts(&hero, true, readonly);
+    selectArtifacts.SetColRows(7, 2);
+    selectArtifacts.SetHSpace(2);
+    selectArtifacts.SetVSpace(2);
+    selectArtifacts.SetContent(hero.GetBagArtifacts());
     selectArtifacts.SetPos(dst_rt.x + 37, dst_rt.y + 95);
-    selectArtifacts.SetInterval(2);
-    selectArtifacts.SetBackgroundSprite(sfb2);
-    selectArtifacts.SetCursorSprite(sfc2);
-    selectArtifacts.SetUseArts32Sprite();
-    if(readonly) selectArtifacts.SetReadOnly();
     selectArtifacts.Redraw();
 
     Button buttonDismiss(dst_rt.x + dst_rt.w / 2 - 160, dst_rt.y + dst_rt.h - 125, ICN::HSBTNS, 0, 1);
@@ -159,7 +125,7 @@ Dialog::answer_t PocketPC::HeroesOpenDialog(Heroes & hero, bool readonly)
     Button buttonPrev(dst_rt.x + 34, dst_rt.y + 200, ICN::TRADPOST, 3, 4);
     Button buttonNext(dst_rt.x + 275, dst_rt.y + 200, ICN::TRADPOST, 5, 6);
 
-    if(castle || readonly)
+    if(hero.inCastle() || readonly)
     {
 	buttonDismiss.Press();
         buttonDismiss.SetDisable(true);
@@ -201,36 +167,37 @@ Dialog::answer_t PocketPC::HeroesOpenDialog(Heroes & hero, bool readonly)
 	    Dialog::YES == Dialog::Message(hero.GetName(), _("Are you sure you want to dismiss this Hero?"), Font::BIG, Dialog::YES | Dialog::NO))
         { return Dialog::DISMISS; }
 
-        // primary click info
-        if(le.MouseClickLeft(ras)) Dialog::Message(_("Attack Skill"), _("Your attack skill is a bonus added to each creature's attack skill."), Font::BIG, Dialog::OK);
-        else
-        if(le.MouseClickLeft(rds)) Dialog::Message(_("Defense Skill"), _("Your defense skill is a bonus added to each creature's defense skill."), Font::BIG, Dialog::OK);
-        else
-        if(le.MouseClickLeft(rps)) Dialog::Message(_("Spell Power"), _("Your spell power determines the length or power of a spell."), Font::BIG, Dialog::OK);
-        else
-        if(le.MouseClickLeft(rks)) Dialog::Message(_("Knowledge"), _("Your knowledge determines how many spell points your hero may have. Under normal cirumstances, a hero is limited to 10 spell points per level of knowledge."), Font::BIG, Dialog::OK);
-
-	if(le.MouseCursor(secskill_bar.GetArea())) secskill_bar.QueueEventProcessing();
+	// skills click
+	if(le.MouseCursor(primskill_bar.GetArea()) && primskill_bar.QueueEventProcessing())
+	{
+            cursor.Show();
+    	    display.Flip();
+        }
+	else
+	if(le.MouseCursor(secskill_bar.GetArea()) && secskill_bar.QueueEventProcessing())
+	{
+            cursor.Show();
+    	    display.Flip();
+        }
 
         // selector troops event
-        if(le.MouseCursor(selectArmy.GetArea()))
-        {
-            if(selectArtifacts.isSelected()) selectArtifacts.Reset();
-	    if(SelectArmyBar::QueueEventProcessing(selectArmy))
-            {
-                cursor.Hide();
-                moraleIndicator.Redraw();
-                luckIndicator.Redraw();
-                cursor.Show();
-        	display.Flip();
-            }
+        if(le.MouseCursor(selectArmy.GetArea()) && selectArmy.QueueEventProcessing())
+	{
+	    if(selectArtifacts.isSelected()) selectArtifacts.ResetSelected();
+    	    moraleIndicator.Redraw();
+            luckIndicator.Redraw();
+	    selectArmy.Redraw();
+            cursor.Show();
+    	    display.Flip();
         }
 
         // selector artifacts event
-        if(le.MouseCursor(selectArtifacts.GetArea()))
-        {
-            if(selectArmy.isSelected()) selectArmy.Reset();
-            SelectArtifactsBar::QueueEventProcessing(selectArtifacts);
+        if(le.MouseCursor(selectArtifacts.GetArea()) && selectArtifacts.QueueEventProcessing())
+	{
+    	    if(selectArmy.isSelected()) selectArmy.ResetSelected();
+	    selectArtifacts.Redraw();
+    	    cursor.Show();
+    	    display.Flip();
         }
 
         if(le.MouseCursor(moraleIndicator.GetArea())) MoraleIndicator::QueueEventProcessing(moraleIndicator);
