@@ -39,9 +39,10 @@
 #include <android/log.h>
 #endif
 
+#define FORMAT_VERSION_2864 0x0B30
 #define FORMAT_VERSION_2850 0x0B22
 #define FORMAT_VERSION_2830 0x0B0E
-#define CURRENT_FORMAT_VERSION FORMAT_VERSION_2850
+#define CURRENT_FORMAT_VERSION FORMAT_VERSION_2864
 #define LAST_FORMAT_VERSION FORMAT_VERSION_2830
 
 enum
@@ -111,12 +112,7 @@ namespace std
  #define VERBOSE(x) std::cout << x << std::endl
 #endif
 
-#ifdef WITH_DEBUG
- #define DEBUG(x, y, z) if(IS_DEBUG(x, y)) VERBOSE(String::GetTime() << ": [" << StringDebug(x) << "]\t" << __FUNCTION__ << ":  " << z)
-#else
- #define DEBUG(x, y, z)
-#endif
-
+#define DEBUG(x, y, z) if(IS_DEBUG(x, y)) VERBOSE(String::GetTime() << ": [" << StringDebug(x) << "]\t" << __FUNCTION__ << ":  " << z)
 #define IS_DEVEL() IS_DEBUG(DBG_DEVEL, DBG_INFO)
 
 bool IS_DEBUG(int name, int level);
@@ -364,7 +360,8 @@ public:
     void SetUnicode(bool);
     void SetPriceLoyaltyVersion(void);
     void SetGameDifficulty(u8);
-    void SetExtGameEvilInterface(bool);
+    void SetEvilInterface(bool);
+    void SetHideInterface(bool);
     void SetBattleGrid(bool);
     void SetBattleMovementShaded(bool);
     void SetBattleMouseShaded(bool);
